/********************************************************************
UDP Windows Application
File Name: UDPWin.cpp

DESCRIPTION
This Win32 console application will send and receive UDP packets


COMMAND LINE PARAMETERS



COMPILILATION INSTRUCTIONS



REVISION HISTORY
2/3/2003	Initial Release


*********************************************************************/

#include <winsock.h>
#include <stdio.h>
#include <stdlib.h>

#define RX_BUFSIZE	(10000)

//----- functions -----
void ProcessCmdLine(int argc, char **argv);
DWORD WINAPI ReceiveUDP(LPVOID lpParam);


//----- global vars -----
WORD dListenPort, dDestPort;
char szDestIPAddr[80];	
char RxBuf[RX_BUFSIZE];


/*-------------------------------------------------------------------
 FUNCTION: ProcessCmdLine

 DESCRIPTION:
 Process command line arguments

 RETURNS: Nothing
 ------------------------------------------------------------------*/
void ProcessCmdLine(int argc, char **argv)
{
	if (argc < 4)
	{
		printf("Usage: UDPWin <listen port> <dest ip> <dest port>\n");
		exit(1);
	}

	dListenPort = (WORD) atoi(argv[1]);
	strncpy(szDestIPAddr, argv[2], 80);
	dDestPort = (WORD) atoi(argv[3]);
}



/*-------------------------------------------------------------------
 FUNCTION: ProcessClient

 DESCRIPTION:
 This function is called as a thread to handle client connections.

 PARAMETERS
 lpParam is a pointer to the client socket handle.

 RETURNS: 32-bit thread exit code
 ------------------------------------------------------------------*/
DWORD WINAPI ReceiveUDP(LPVOID lpParam)
{
	SOCKET s = (SOCKET)lpParam;
	struct sockaddr_in sinFrom;

	while (1)
	{
		printf("Listening on port: %d....\n", dListenPort);

		/*
		  The recvfrom( ) API function has the following parameters:

		  recvfrom(SOCKET s,	// socket to receive from
					char *buf,	// point to buffer to store rec'd data
					size_t len, // length of buffer or # of bytes to read
					int flags,	// flags (normally use 0)
					struct sockaddr *from,	// client data returned
					int *fromlen	// size of client data struct
					)
		*/
		int len = sizeof(sinFrom);
		int bytes = recvfrom(s, RxBuf, RX_BUFSIZE, 0, (struct sockaddr *)&sinFrom, &len );
		if (bytes == SOCKET_ERROR)
		{
			printf("recvfrom() error: %d\n", WSAGetLastError());
			break;
		}

		printf("Received packet from: %s:%d\n",
			inet_ntoa(sinFrom.sin_addr), ntohs(sinFrom.sin_port));
		printf("Bytes read: %d: \"", bytes);
		for (int i=0; i < bytes; i++)
			printf("%c", RxBuf[i]);
		printf("\"\n");

	}
	return 0;
}




/*-------------------------------------------------------------------
 FUNCTION: main

 DESCRIPTION:
 Main routine to start network services and listen for connections.

 RETURNS: integer exit code
 ------------------------------------------------------------------*/
int main(int argc, char **argv)
{
	SOCKET	sListen, sSend;
	WSADATA	WSAData;
	struct sockaddr_in sinListen, sinSend;

	ProcessCmdLine(argc, argv);

	if (WSAStartup( MAKEWORD(1,1), &WSAData) != 0)
	{
		printf("Error loading Winsock\n");
		return 1;
	}
	
	// Create socket to listen for UDP datagrams
	sListen = socket(AF_INET, SOCK_DGRAM, 0);
	if (sListen == SOCKET_ERROR)
	{
		printf("Error creating sListen socket: %d\n", WSAGetLastError());
		return 1;
	}

	// Setup listen address structure
	sinListen.sin_addr.s_addr = htonl(INADDR_ANY);
	sinListen.sin_family = AF_INET;
	sinListen.sin_port = htons(dListenPort);

	// Bind socket to listen address
	if (bind(sListen, (struct sockaddr *) &sinListen, sizeof(sinListen)) == SOCKET_ERROR)
	{
		printf("bind() error for sListen: %d\n", WSAGetLastError());
		return 1;
	}

	/*
	Create and start thread to receive UDP packets.

	HANDLE CreateThread( 
			LPSECURITY_ATTRIBUTES lpThreadAttributes,  // pointer to thread security attributes 
			DWORD dwStackSize,  // initial thread stack size, in bytes 
			LPTHREAD_START_ROUTINE lpStartAddress, 	// pointer to thread function 
			LPVOID lpParameter, 					// argument for new thread 
			DWORD dwCreationFlags,					// creation flags 
			LPDWORD lpThreadId						// pointer to returned thread identifier 
			); 
	*/
	DWORD  dwThreadId;
	HANDLE hThread;
	hThread = CreateThread(NULL, 0, ReceiveUDP, (LPVOID) sListen, 0, &dwThreadId);

	// Create socket to send UDP datagrams
	sSend = socket(AF_INET, SOCK_DGRAM, 0);
	if (sSend == INVALID_SOCKET)
	{
		printf("Error creating sSend socket: %d\n", WSAGetLastError());
		return 1;
	}

	// name the socket
	sinSend.sin_family = AF_INET;
	sinSend.sin_addr.s_addr = inet_addr(szDestIPAddr);
	sinSend.sin_port = htons(dDestPort);
	
	printf("Enter data to send to %s:%d\n", szDestIPAddr, dListenPort);

	while (1)
	{
		/*
		  The sendto function is used to write outgoing data on a socket. 
		  For message-oriented sockets, care must be taken not to exceed 
		  the maximum packet size of the underlying subnets. You can get
		  the size information from the getsockopt() API call. See the
		  microsoft documentation for more information.

		  The sendto( ) API function has the following parameters:

			int sendto(
			  SOCKET s,
			  const char* buf,	// buffer containing the data to be transmitted
			  int len,			// length of data in buf
			  int flags,		// indicator specifying the way in which the call is made
			  const struct sockaddr* to,	// optional ptr to dest address structure
			  int tolen			// size of address structure in bytes
			);
		*/
		char TxBuf[512];
		gets(TxBuf);
		if (TxBuf[0] == '\0')
			break;
		else
		{
			int len = sizeof(sinSend);
			sendto(sSend, TxBuf, strlen(TxBuf), 0, (struct sockaddr *)&sinSend, len );
			if (sSend == INVALID_SOCKET)
			{
				printf("sendto() error: %d\n", WSAGetLastError());
				break;
			}

			printf("Sent packet to: %s:%d\n",
				inet_ntoa(sinSend.sin_addr), ntohs(sinSend.sin_port));
		}
	}

	closesocket(sListen);
	closesocket(sSend);
	WSACleanup();
	return 0;
}







